@@ -10,7 +10,7 @@ use tap::Tap;
 use crate::{
     containers::{List, Map, Symbol},
     context::{Ctx, CtxLocation, CtxResult, ModuleId, ToCtx, ToCtxErr},
-    grammar::BinOp,
+    grammar::{BinOp, UniOp},
 };
 
 use super::{RawConstExpr, RawDefn, RawExpr, RawProgram, RawTypeBind, RawTypeExpr};
@@ -349,43 +349,66 @@ fn parse_expr(pair: Pair<Rule>, source: ModuleId) -> Ctx<RawExpr> {
         Rule::rel_expr
         | Rule::add_expr
         | Rule::mult_expr
+        | Rule::uni_expr
         | Rule::logic_expr
         | Rule::bitlogic_expr
         | Rule::shift_expr => {
             let mut children: Vec<_> = pair.into_inner().collect();
-            let mut toret = parse_expr(children.remove(0), source);
-            for pair in children.chunks_exact(2) {
-                if let [op, child] = pair {
-                    toret = RawExpr::BinOp(
-                        match op.as_rule() {
-                            Rule::add => BinOp::Add,
-                            Rule::sub => BinOp::Sub,
-                            Rule::mul => BinOp::Mul,
-                            Rule::div => BinOp::Div,
-                            Rule::modulo => BinOp::Mod,
-                            Rule::equal => BinOp::Eq,
-                            Rule::append => BinOp::Append,
-                            Rule::land => BinOp::Land,
-                            Rule::lor => BinOp::Lor,
-                            Rule::le => BinOp::Le,
-                            Rule::lt => BinOp::Lt,
-                            Rule::ge => BinOp::Ge,
-                            Rule::gt => BinOp::Gt,
-                            Rule::band => BinOp::Band,
-                            Rule::bor => BinOp::Bor,
-                            Rule::bxor => BinOp::Bxor,
-                            Rule::lshift => BinOp::Lshift,
-                            Rule::rshift => BinOp::Rshift,
-                            _ => unreachable!(),
-                        }
-                        .with_ctx(p2ctx(child, source)),
-                        toret.clone(),
-                        parse_expr(child.clone(), source),
-                    )
-                    .with_ctx(ctx);
+
+            if children.len() == 1 {
+                parse_expr(children.remove(0), source)
+            }
+            else if children.len() == 2 {
+                let c1 = children.remove(0);
+                match c1.as_rule() {
+                    Rule::bnot => {
+                        let child = children.remove(0);
+                        let e = parse_expr(child.clone(), source);
+                        RawExpr::UniOp(UniOp::Bnot.with_ctx(ctx), e).with_ctx(ctx)
+                    }
+                    Rule::lnot => {
+                        let child = children.remove(0);
+                        let e = parse_expr(child.clone(), source);
+                        RawExpr::UniOp(UniOp::Lnot.with_ctx(ctx), e).with_ctx(ctx)
+                    }
+                    _ => unreachable!(),
                 }
             }
-            toret
+            else {
+                let mut toret = parse_expr(children.remove(0), source);
+                for pair in children.chunks_exact(2) {
+                    if let [op, child] = pair {
+                        toret = RawExpr::BinOp(
+                            match op.as_rule() {
+                                Rule::add => BinOp::Add,
+                                Rule::sub => BinOp::Sub,
+                                Rule::mul => BinOp::Mul,
+                                Rule::div => BinOp::Div,
+                                Rule::modulo => BinOp::Mod,
+                                Rule::equal => BinOp::Eq,
+                                Rule::append => BinOp::Append,
+                                Rule::land => BinOp::Land,
+                                Rule::lor => BinOp::Lor,
+                                Rule::le => BinOp::Le,
+                                Rule::lt => BinOp::Lt,
+                                Rule::ge => BinOp::Ge,
+                                Rule::gt => BinOp::Gt,
+                                Rule::band => BinOp::Band,
+                                Rule::bor => BinOp::Bor,
+                                Rule::bxor => BinOp::Bxor,
+                                Rule::lshift => BinOp::Lshift,
+                                Rule::rshift => BinOp::Rshift,
+                                _ => unreachable!(),
+                            }
+                            .with_ctx(p2ctx(&op, source)),
+                            toret.clone(),
+                            parse_expr(child.clone(), source),
+                        )
+                        .with_ctx(ctx);
+                    }
+                }
+                toret
+            }
         }
         Rule::extern_call_expr => {
             let mut children = pair.into_inner();
