@@ -74,6 +74,7 @@ impl<TVar: Variable, CVar: Variable> Expr<TVar, CVar> {
 #[derive(Debug, Clone)]
 pub enum ExprInner<TVar: Variable, CVar: Variable> {
     BinOp(BinOp, Arc<Expr<TVar, CVar>>, Arc<Expr<TVar, CVar>>),
+    UniOp(UniOp, Arc<Expr<TVar, CVar>>),
     If(
         Arc<Expr<TVar, CVar>>,
         Arc<Expr<TVar, CVar>>,
@@ -133,6 +134,12 @@ impl<TVar: Variable, CVar: Variable> ExprInner<TVar, CVar> {
     }
 }
 
+/// Unary operator
+#[derive(Clone, Copy, Debug)]
+pub enum UniOp {
+    Bnot
+}
+
 /// Binary operator
 #[derive(Clone, Copy, Debug)]
 pub enum BinOp {
